// import "colors";
import * as css from "css-tree";
import * as fs from "fs";
import { Class } from "./Class";
import { TailwindConfig } from "./TailwindConfig";

export class Parser {
  readonly inputFilePath: string;
  readonly config: TailwindConfig;
  constructor(inputFilePath: string, config: TailwindConfig) {
    this.inputFilePath = inputFilePath;
    this.config = config;
  }

  private ruleToClass(child) {
    const rule = child as css.Rule;
    if (rule.prelude.type === "SelectorList") {
      const selectorList = rule.prelude as css.SelectorList;
      // tailwind uses only one selector
      const selector = selectorList.children[0].children[0];
      if (
        selector.type === "ClassSelector" ||
        selector.type === "TypeSelector"
      ) {
        // e.g. text-bg-red
        const classSelector = selector as css.ClassSelector;
        const clazz: Class = { name: "", properties: [] };
        clazz.name = classSelector.name.split("\\:").join(":");
        for (const blockChildren of Array.from(
          rule.block.children as any
        ) as any[]) {
          if (blockChildren.type === "Declaration") {
            const { property, value } = blockChildren as css.Declaration;
            clazz.properties.push({
              name: property,
              value: this.getValues(property, value as css.Value)
            });
          } else {
            // console.log(blockChildren);
          }
        }
        console.log("returning", clazz.name);
        return clazz;
      } else if (selector.type === "TypeSelector") {
        const typeSelector = selector as css.TypeSelector;
      } else if (selector.type === "AttributeSelector") {
        const typeSelector = selector as css.AttributeSelector;
        // e.g. html, input
        return;
      } else if (selector.type === "PseudoElementSelector") {
        const typeSelector = selector as css.PseudoElementSelector;
        // e.g. ::-webkit-file-upload-button
        return;
      } else {
        throw new Error("unknown selector type " + selector.type);
      }
    }
  }
  private getValues(property: string, value: css.Value): any {
    const children = new Array(value.children[0]);
    if (children.length === 1) {
      const child = children[0];
      return this.getValue(child);
    } else {
      // console.log(children);
    }
  }
  private getValue(child) {
    if (child.type === "Percentage") {
      return `'${child.value}%'`;
    } else if (child.type === "HexColor") {
      return `'#${child.value}'`;
    } else if (child.type === "Identifier") {
      return `'${child.name}'`;
    } else if (child.type === "Number") {
      return child.value;
    } else if (child.type === "Dimension") {
      const dimension = child as css.Dimension;
      return `toNative(${dimension.value as any}, "${dimension.unit}")`;
    } else {
      //   // children; //?.$
    }
  }
  parse() {
    const content = fs.readFileSync(this.inputFilePath);
    const ast = css.parse(content.toString());
    const obj = css.toPlainObject(ast);

    const children = new Array((obj as any).children);
    const result = new Array<Class>();

    for (const child of children[0]) {
      if (child.type === "Rule") {
        const clazz = this.ruleToClass(child);
        if (clazz !== undefined) {
          result.push(clazz);

          let breakpoint = "";
          if (clazz.name.indexOf("\\:") !== -1) {
            breakpoint = clazz.name.substring(0, clazz.name.indexOf("\\:"));
            clazz.name = clazz.name.substring(
              breakpoint.length,
              clazz.name.length
            );
          }

          if (!breakpoint) {
            // add platform specific classes
            if (this.config.platforms) {
              this.config.platforms.map(platform =>
                result.push({
                  name: `${platform}:${clazz.name}`,
                  properties: clazz.properties,
                  condition: `true` // `(require("react-native").OS === "${platform}")`
                })
              );
            }
            if (this.config.observeInternetConnectivity) {
              // add internet connectivity classes
              this.config.platforms.map(platform =>
                result.push({
                  name: `online:${clazz.name}`,
                  properties: clazz.properties,
                  condition: `true` // (require("react-native-web-tailwind").isOnline())
                })
              );
              this.config.platforms.map(platform =>
                result.push({
                  name: `offline:${clazz.name}`,
                  properties: clazz.properties,
                  condition: `true` // `(!require("react-native-web-tailwind").isOnline())`
                })
              );
            }
          }
        }
      } else if (child.type === "Atrule") {
        const atrule = child as css.Atrule;
        if (atrule.name === "media") {
          let query = "";
          const mediaQueryList = new Array(
            (atrule.prelude as css.AtrulePrelude).children
          )[0][0] as css.MediaQueryList;
          for (const _mediaQuery of new Array(mediaQueryList.children)) {
            for (const { name, value } of _mediaQuery[0].children) {
              query = `mediaQuery("${name}: ${value.value}px")`; // { name, value: this.getValue(value) };
            }
          }
          if (atrule.block) {
            for (const rule of Array.from(atrule.block.children as any)) {
              const clazz = this.ruleToClass(rule);
              if (clazz) {
                clazz.condition = `true`; //  query;
                result.push(clazz);
              } else {
                console.log("is null");
              }
            }
          }
        } else {
          console.log("noo");
        }
      } else {
        if (child.type !== "Comment") {
          console.log("unknown css node", child);
        }
      }
    }

    const f = require("fs").createWriteStream(".log.txt");
    f.write(JSON.stringify(result, null, 2));
    f.close();
    return result;
  }
}
